import { useEffect, useState } from 'react';
import { ShutterAction, ShutterFixedValue, ShutterOngoingAction, ShutterRanges } from '../components/shutterModels';

const shutterActions: { [key: string]: ShutterAction } = {
  CLOSE: 'CLOSE',
  DOWN: 'DOWN',
  OPEN: 'OPEN',
  STOP: 'STOP',
  UP: 'UP',
};

export const shutterRanges: { [key: string]: ShutterRanges } = {
  R1: 'R1',
  R2: 'R2',
  R3: 'R3',
  R4: 'R4',
};

const shutterFixedValues: { [key: string]: ShutterFixedValue } = {
  V0: 0,
  V25: 25,
  V50: 50,
  V75: 75,
  V100: 100,
};

const getShutterRange = (
  value: number,
  action: ShutterAction = shutterActions.DOWN,
): ShutterRanges => {
  const isUnder25 =
    action === shutterActions.DOWN
      ? value <= shutterFixedValues.V25
      : value < shutterFixedValues.V25;
  const isBetween25And50 =
    action === shutterActions.DOWN
      ? value > shutterFixedValues.V25 && value <= shutterFixedValues.V50
      : value >= shutterFixedValues.V25 && value < shutterFixedValues.V50;
  const isBetween50And75 =
    action === shutterActions.DOWN
      ? value > shutterFixedValues.V50 && value <= shutterFixedValues.V75
      : value >= shutterFixedValues.V50 && value < shutterFixedValues.V75;
  const isAbove75 =
    action === shutterActions.DOWN
      ? value > shutterFixedValues.V75
      : value >= shutterFixedValues.V75;

  if (isUnder25) {
    return shutterRanges.R1;
  }
  if (isBetween25And50) {
    return shutterRanges.R2;
  }
  if (isBetween50And75) {
    return shutterRanges.R3;
  }
  if (isAbove75) {
    return shutterRanges.R4;
  }
  return shutterRanges.R1;
};

const shutterActionsState: {
  [x: string]: (action: ShutterAction) => ShutterFixedValue;
} = {
  [shutterRanges.R1]: (action: ShutterAction): ShutterFixedValue => {
    return action === shutterActions.DOWN
      ? shutterFixedValues.V0
      : shutterFixedValues.V25;
  },
  [shutterRanges.R2]: (action: ShutterAction): ShutterFixedValue => {
    return action === shutterActions.DOWN
      ? shutterFixedValues.V25
      : shutterFixedValues.V50;
  },
  [shutterRanges.R3]: (action: ShutterAction): ShutterFixedValue => {
    return action === shutterActions.DOWN
      ? shutterFixedValues.V50
      : shutterFixedValues.V75;
  },
  [shutterRanges.R4]: (action: ShutterAction): ShutterFixedValue => {
    return action === shutterActions.DOWN
      ? shutterFixedValues.V75
      : shutterFixedValues.V100;
  },
};

const getValueToSettingAction = (
  value: number,
  action: ShutterAction,
): ShutterFixedValue => {
  const shutterRange = getShutterRange(value, action);
  const getActionValue = shutterActionsState[shutterRange];
  return getActionValue(action);
};

export const useShutter = (inputValue = 0) => {
  const [shutterOutputValue, setShutterOutputValue] = useState<number>(inputValue); 
  const [shutterOngoingAction, setShutterOngoingAction] = useState<ShutterOngoingAction>(null);
  
  const shutterAbsoluteAperture: {[key: string] : boolean}  = {
    FULLY_OPENED: inputValue === 100,
    FULLY_CLOSED: inputValue === 0,
  };

  useEffect(() => {
    setShutterOutputValue(inputValue);
  }, [inputValue]);

  const onRange = (value: number): ShutterRanges => {
    return getShutterRange(value);
  };

  const onShutterAction = (action: ShutterAction): number | ShutterFixedValue => {
    const outputValueByAction: { [key: string]: number | ShutterFixedValue } = {
      [shutterActions.STOP]: shutterOutputValue,
      [shutterActions.OPEN]: shutterFixedValues.V100,
      [shutterActions.CLOSE]: shutterFixedValues.V0,
      [shutterActions.UP]: getValueToSettingAction(inputValue, action),
      [shutterActions.DOWN]: getValueToSettingAction(inputValue, action),
    };
    setShutterOngoingAction(action);
    return outputValueByAction[action];
  };

  return { 
    shutterOutputValue: shutterOutputValue as ShutterFixedValue, 
    shutterOngoingAction, 
    shutterAbsoluteAperture, 
    onShutterAction, 
    onRange, 
  };
};
